@@ -13,24 +13,26 @@ use crate::*;
 
 pub fn handle(
     params: &mut Parameters,
-    options: &clap::ArgMatches,
+    matches: &clap::ArgMatches,
+    sub_matches: &clap::ArgMatches,
     output_params: DisplayParams,
 ) -> CliResult<()> {
-    let cumulative = options.is_present("cumulative");
-    let resilent = options.is_present("resilient");
-    if options.value_of("stage").is_some() {
+    let cumulative = sub_matches.is_present("cumulative");
+    let resilent = sub_matches.is_present("resilient");
+    let allow_random_input: bool = matches.is_present("allow-random-input");
+    if sub_matches.value_of("stage").is_some() {
         // --with is by pipeline and put in params
-        return handle_reference_stage(cumulative, params, &output_params);
-    } else if let Some(npz) = options.value_of("npz") {
-        return handle_npz(cumulative, npz, params, &output_params);
-    } else if options.is_present("twice") {
-        return handle_twice(cumulative, params, &output_params);
+        return handle_reference_stage(cumulative, params, &output_params, allow_random_input);
+    } else if let Some(npz) = sub_matches.value_of("npz") {
+        return handle_npz(cumulative, npz, params, &output_params, allow_random_input);
+    } else if sub_matches.is_present("twice") {
+        return handle_twice(cumulative, params, &output_params, allow_random_input);
     }
-    if let Some(pbdir) = options.value_of("pbdir") {
-        return handle_pbdir(cumulative, pbdir, params, &output_params);
+    if let Some(pbdir) = sub_matches.value_of("pbdir") {
+        return handle_pbdir(cumulative, pbdir, params, &output_params, allow_random_input);
     }
-    if options.is_present("tf") {
-        return handle_tensorflow(cumulative, resilent, params, &output_params);
+    if sub_matches.is_present("tf") {
+        return handle_tensorflow(cumulative, resilent, params, &output_params, allow_random_input);
     }
     bail!("No comparison target found")
 }
@@ -41,6 +43,7 @@ pub fn handle_tensorflow(
     _resilient: bool,
     _params: &mut Parameters,
     _output_params: &DisplayParams,
+    _allow_random_input: bool,
 ) -> CliResult<()> {
     bail!("`tf` feature is required for this to work");
 }
@@ -51,6 +54,7 @@ pub fn handle_tensorflow(
     resilient: bool,
     params: &mut Parameters,
     output_params: &DisplayParams,
+    allow_random_input: bool,
 ) -> CliResult<()> {
     let tract = &params.tract_model;
     let mut tf = params.tf_model.take().unwrap();
@@ -123,13 +127,15 @@ pub fn handle_npz(
     npz: &str,
     params: &Parameters,
     output_params: &DisplayParams,
+    allow_random_input: bool,
 ) -> CliResult<()> {
     use tensor::for_npz;
     let mut npz = ndarray_npy::NpzReader::new(std::fs::File::open(npz)?)?;
     let mut values: HashMap<String, Vec<CliResult<Arc<Tensor>>>> = HashMap::new();
+    let multiturn = npz.names()?.iter().any(|n| n.starts_with("turn_0/"));
     for name in npz.names()? {
         if let Ok(value) = for_npz(&mut npz, &name) {
-            if params.multiturn {
+            if multiturn {
                 let name = name
                     .split("/")
                     .nth(1)
@@ -152,7 +158,8 @@ pub fn handle_npz(
         m,
         &values,
         &params,
-        output_params
+        output_params,
+        allow_random_input
     ))
 }
 
@@ -172,6 +179,7 @@ pub fn handle_pbdir(
     pbdir: &str,
     params: &Parameters,
     output_params: &DisplayParams,
+    allow_random_input: bool,
 ) -> CliResult<()> {
     let mut values: HashMap<String, Vec<CliResult<Arc<Tensor>>>> = HashMap::new();
     for entry in fs::read_dir(pbdir)? {
@@ -186,7 +194,8 @@ pub fn handle_pbdir(
         m,
         &values,
         &params,
-        output_params
+        output_params,
+        allow_random_input
     ))
 }
 
@@ -194,23 +203,25 @@ pub fn handle_twice(
     cumulative: bool,
     params: &Parameters,
     output_params: &DisplayParams,
+    allow_random_input: bool,
 ) -> CliResult<()> {
     let reference_model =
         params.tract_model.downcast_ref::<TypedModel>().context("Only work with a typed model")?;
-    handle_with_model(cumulative, params, output_params, &reference_model)
+    handle_with_model(cumulative, params, output_params, &reference_model, allow_random_input)
 }
 
 pub fn handle_reference_stage(
     cumulative: bool,
     params: &Parameters,
     output_params: &DisplayParams,
+    allow_random_input: bool,
 ) -> CliResult<()> {
     let reference_model =
         params.reference_model.as_ref().context("No reference model. need --with ?")?;
     let reference_model = reference_model
         .downcast_ref::<TypedModel>()
         .context("Only work with a typed reference model")?;
-    handle_with_model(cumulative, params, output_params, &reference_model)
+    handle_with_model(cumulative, params, output_params, &reference_model, allow_random_input)
 }
 
 pub fn handle_with_model(
@@ -218,12 +229,13 @@ pub fn handle_with_model(
     params: &Parameters,
     output_params: &DisplayParams,
     reference_model: &TypedModel,
+    allow_random_input: bool,
 ) -> CliResult<()> {
     let mut values: HashMap<String, Vec<CliResult<Arc<Tensor>>>> = HashMap::new();
 
     let plan = SimplePlan::new(reference_model)?;
     let mut state = SimpleState::new(plan)?;
-    for inputs in crate::tensor::retrieve_or_make_inputs(reference_model, params)? {
+    for inputs in crate::tensor::retrieve_or_make_inputs(reference_model, params, allow_random_input)? {
         state.run_plan_with_eval(inputs, |session, state, node, input| -> TractResult<_> {
             let result: TVec<Arc<Tensor>> = tract_core::plan::eval(session, state, node, input)?;
             if node.outputs.len() == 1 {
@@ -244,7 +256,8 @@ pub fn handle_with_model(
         m,
         &values,
         params,
-        output_params
+        output_params,
+        allow_random_input
     ))
 }
 
@@ -254,6 +267,7 @@ pub fn compare<F, O>(
     all_values: &HashMap<String, Vec<CliResult<Arc<Tensor>>>>,
     params: &Parameters,
     output_params: &DisplayParams,
+    allow_random_input: bool,
 ) -> CliResult<()>
 where
     F: Fact + Clone + for<'a> From<&'a Tensor> + Hash,
@@ -275,7 +289,7 @@ where
     }
     let all_values: HashMap<String, &Vec<CliResult<Arc<Tensor>>>> =
         all_values.iter().map(|(k, v)| (canonic(k), v)).collect();
-    for (turn, inputs) in tensor::retrieve_or_make_inputs(tract, params)?.into_iter().enumerate() {
+    for (turn, inputs) in tensor::retrieve_or_make_inputs(tract, params, allow_random_input)?.into_iter().enumerate() {
         state.run_plan_with_eval(
             inputs,
             |session_state, state, node, input| -> TractResult<TVec<Arc<Tensor>>> {
@@ -293,6 +307,21 @@ where
                             .outlet_label((node.id, ix).into())
                             .and_then(get_value)
                             .or_else(|| get_value(&node.name))
+                            .map(|t| {
+                                // big ugly, doing this for the npz case, where model is not
+                                // exploited at all, so quantization information is erased.
+                                let needed_type =
+                                    node.outputs[ix].fact.to_typed_fact().unwrap().datum_type;
+                                if needed_type != t.datum_type()
+                                    && needed_type.unquantized() == t.datum_type().unquantized()
+                                {
+                                    let mut t = t.into_tensor();
+                                    unsafe { t.set_datum_type(needed_type) };
+                                    t.into_arc_tensor()
+                                } else {
+                                    t
+                                }
+                            })
                     })
                     .collect();
                 let mut tested = None;
