@@ -23,7 +23,8 @@ pub fn handle(
             .tract_model
             .downcast_ref::<TypedModel>()
             .context("Can only profile typed models")?;
-        crate::profile::profile(model, bench_limits, &mut annotations, params)?;
+        let allow_random_input = matches.is_present("allow-random-input");
+        crate::profile::profile(model, bench_limits, &mut annotations, params, allow_random_input)?;
     }
 
     if let Some(asserts) = &params.assertions.assert_output_facts {
@@ -49,7 +50,7 @@ pub fn handle(
             rename_outputs(&mut typed, sub_matches)?;
             let file = std::fs::File::create(path)?;
             let encoder = flate2::write::GzEncoder::new(file, flate2::Compression::default());
-            nnef.write_to_tar(&typed, encoder)?;
+            nnef.write_to_tar(&typed, encoder).context("Writting model to tar")?;
         } else {
             bail!("Only typed model can be dumped")
         }
@@ -60,7 +61,7 @@ pub fn handle(
         if let Some(mut typed) = model.downcast_ref::<TypedModel>().cloned() {
             rename_outputs(&mut typed, sub_matches)?;
             let file = std::fs::File::create(path)?;
-            nnef.write_to_tar(&typed, file)?;
+            nnef.write_to_tar(&typed, file).context("Writting model to tar")?;
         } else {
             bail!("Only typed model can be dumped")
         }
