@@ -60,8 +60,6 @@ pub const STAGES: &'static [&'static str] = &[
     "incorporate",
     "type",
     "declutter",
-    "concretize-stream-dim",
-    "concretize-stream-dim-declutter",
     "pulse",
     "pulse-to-type",
     "pulse-declutter",
@@ -88,6 +86,7 @@ fn main() -> tract_core::anyhow::Result<()> {
 
         .arg(arg!(--"const-input" [const_input] ... "Treat input as a Const (by name), retaining its value."))
         .arg(arg!(--"input-bundle" [input_bundle] "Path to an input container (.npz)"))
+        .arg(arg!(--"allow-random-input" "WIll use random generated input"))
 
         .arg(arg!(--"kaldi-adjust-final-offset" [frames] "Adjust value of final offset in network (for reproducibility)"))
         .arg(arg!(--"kaldi-downsample" [frames] "Add a subsampling to output on axis 0"))
@@ -108,7 +107,6 @@ fn main() -> tract_core::anyhow::Result<()> {
         .arg(arg!(--recursive "Apply to sub graphes"))
         .arg(arg!(--proto "Keep proto model around after parse"))
         .arg(arg!(--determinize "Enforce a seed in random operator"))
-        .arg(arg!(--multiturn "Switch run and compare to multiturn, session state preserving"))
         .arg(arg!(--partial "Before analyse, eliminate dead branches"))
 
         .arg(arg!(--pass [STAGE] "Pass to stop preprocessing after.").possible_values(STAGES))
@@ -124,7 +122,6 @@ fn main() -> tract_core::anyhow::Result<()> {
 
         .arg(arg!(-O --optimize "Optimize before running"))
         .arg(arg!(--pulse [PULSE] "Translate to pulse network"))
-        .arg(arg!(--"concretize-stream-dim" [VALUE] "Replace streaming dim by a concrete value"))
 
         .arg(arg!(--"machine-friendly" "Machine friendly output"))
 
@@ -443,19 +440,19 @@ fn handle(matches: clap::ArgMatches, probe: Option<&Probe>) -> CliResult<()> {
     match matches.subcommand() {
         Some(("bench", m)) => {
             need_optimisations = true;
-            bench::handle(&params, &BenchLimits::from_clap(&m)?, probe)
+            bench::handle(&params, &matches, &BenchLimits::from_clap(&m)?, probe)
         }
 
         Some(("criterion", _)) => {
             need_optimisations = true;
-            bench::criterion(&params)
+            bench::criterion(&params, &matches)
         }
 
         Some(("compare", m)) => {
-            compare::handle(&mut params, &m, display_params_from_clap(&matches, m)?)
+            compare::handle(&mut params, &matches, &m, display_params_from_clap(&matches, m)?)
         }
 
-        Some(("run", m)) => run::handle(&params, m),
+        Some(("run", m)) => run::handle(&params, &matches, m),
 
         #[cfg(feature = "pulse")]
         Some(("stream-check", m)) => {
