@@ -1,4 +1,5 @@
 use reqwest::Url;
+use std::any::Any;
 use std::io::Read;
 use std::path::PathBuf;
 use std::str::FromStr;
@@ -107,7 +108,6 @@ pub struct Parameters {
     pub assertions: Assertions,
 
     pub machine_friendly: bool,
-    pub multiturn: bool,
 }
 
 #[cfg(feature = "tf")]
@@ -403,6 +403,7 @@ impl Parameters {
         matches: &clap::ArgMatches,
         location: &ModelLocation,
         onnx_tc: bool,
+        ignore_input_shape: bool,
     ) -> CliResult<HashMap<String, Vec<Arc<Tensor>>>>
     where
         F: std::fmt::Debug + Clone + Hash + Fact + for<'a> TryFrom<&'a InferenceFact, Error = E>,
@@ -414,7 +415,7 @@ impl Parameters {
             + Hash
             + Send
             + Sync,
-        Graph<F, O>: SpecialOps<F, O> + Send,
+        Graph<F, O>: SpecialOps<F, O> + Send + Any,
         tract_core::ops::konst::Const: Into<O>,
         E: std::fmt::Debug,
     {
@@ -465,7 +466,8 @@ impl Parameters {
                     }
                 }
                 let input_outlets = raw_model.input_outlets()?.to_vec();
-                let last_turn = if matches.is_present("multiturn") {
+                let multiturn = npz.names()?.iter().any(|npy| npy.starts_with("turn_0/"));
+                let last_turn = if multiturn {
                     npz.names()?
                         .iter()
                         .map(|name| {
@@ -487,7 +489,7 @@ impl Parameters {
                     let mut values = vec![];
                     let name = raw_model.node(input.node).name.clone();
                     for turn in 0..=last_turn {
-                        let filename = if matches.is_present("multiturn") {
+                        let filename = if multiturn {
                             format!("turn_{}/{}", turn, name)
                         } else {
                             name.to_string()
@@ -512,8 +514,10 @@ impl Parameters {
                             unsafe {
                                 t.set_datum_type(dt);
                             }
-                            let fact = InferenceFact::dt_shape(dt, shape);
-                            raw_model.set_input_fact(ix, (&fact).try_into().unwrap())?;
+                            if !ignore_input_shape {
+                                let fact = InferenceFact::from(dt.fact(shape));
+                                raw_model.set_input_fact(ix, (&fact).try_into().unwrap())?;
+                            }
                             values.push(t.into_arc_tensor());
                         }
                     }
@@ -577,9 +581,6 @@ impl Parameters {
         let pulse: Option<usize> =
             matches.value_of("pulse").map(|s| s.parse::<usize>()).transpose()?;
         #[cfg(feature = "pulse")]
-        let concretize_stream_dim: Option<usize> =
-            matches.value_of("concretize-stream-dim").map(|s| s.parse()).transpose()?;
-
         let stop_at = matches.value_of("pass").unwrap_or(if matches.is_present("optimize") {
             "optimize"
         } else {
@@ -615,7 +616,7 @@ impl Parameters {
                     let block: &dyn Fn(_) -> CliResult<_> = &$block;
                     let owned_model =
                         Arc::try_unwrap(from).unwrap_or_else(|from| from.as_ref().clone());
-                    match block(owned_model) {
+                    match block(owned_model).context(concat!("Error at stage ", $name)) {
                         Ok(it) => {
                             $to = Some(Arc::new(it));
                         }
@@ -623,7 +624,7 @@ impl Parameters {
                             if let Some(last_model) = last_model.take() {
                                 return Err(ModelBuildingError(last_model, e.into()))?;
                             } else {
-                                return Err(e);
+                                return Err(e)?;
                             }
                         }
                     }
@@ -670,10 +671,7 @@ impl Parameters {
         });
         #[cfg(feature = "pulse")]
         {
-            if let Some(dim) = concretize_stream_dim {
-                stage!("concretize-stream-dim", typed_model -> typed_model, |m:TypedModel| Ok(m.concretize_dims(&SymbolValues::default().with(stream_symbol(), dim as _))?));
-                stage!("concretize-stream-dim-declutter", typed_model -> typed_model, |m:TypedModel| m.into_decluttered());
-            } else if let Some(pulse) = pulse {
+            if let Some(pulse) = pulse {
                 stage!("pulse", typed_model -> pulsed_model, |m:TypedModel| Ok(PulsedModel::new(&m, pulse)?));
                 stage!("pulse-to-type", pulsed_model -> typed_model, |m:PulsedModel| Ok(m.into_typed()?));
                 stage!("pulse-declutter", typed_model -> typed_model, |m:TypedModel| Ok(m.into_decluttered()?));
@@ -683,9 +681,9 @@ impl Parameters {
             stage!("nnef-cycle", typed_model -> typed_model, |m:TypedModel| {
                 let nnef = super::nnef(&matches);
                 let mut vec = vec!();
-                nnef.write(&m, &mut vec)?;
+                nnef.write(&m, &mut vec).context("Serializing")?;
                 info!("Dumped, now reloading...");
-                Ok(nnef.model_for_read(&mut &*vec)?)
+                Ok(nnef.model_for_read(&mut &*vec).context("Deserializing")?)
             });
             stage!("nnef-declutter", typed_model -> typed_model, |m:TypedModel| Ok(m.into_decluttered()?));
         }
@@ -821,12 +819,14 @@ impl Parameters {
             dispatch_model_mut_no_pulse!(raw_model, |m| Self::kaldi_context(m, left, right))?;
         }
 
+        let ignore_input_facts = raw_model.is::<TypedModel>();
         let input_values = dispatch_model_mut_no_pulse!(raw_model, |m| Self::inputs(
             m,
             &mut assertions,
             matches,
             &filename,
-            onnx_tc
+            onnx_tc,
+            ignore_input_facts
         ))?;
 
         if matches.is_present("partial") {
@@ -856,7 +856,6 @@ impl Parameters {
                 input_values,
                 assertions,
                 machine_friendly: matches.is_present("machine-friendly"),
-                multiturn: matches.is_present("multiturn"),
             }
         })
     }
